#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from enum import Enum

class ActionType(Enum): # pylint: disable=too-few-public-methods
    Fold = 'f'
    Check = 'x'
    Call = 'c'
    CallAi = 'cai'
    Bet = 'b'
    BetAi = 'bai'
    Raise = 'r'
    RaiseAi = 'rai'
    Post = 'p'
    Uncalled = 'u'

class Action(object): # pylint: disable=too-few-public-methods
    def __init__(self, atype, avalue=0):
        self.type = atype
        self.value = avalue
        self.player = None

    def is_call(self):
        return self.type in [ActionType.Call, ActionType.CallAi]

    def is_raise(self):
        return self.type in [ActionType.Raise, ActionType.RaiseAi]

    def is_voluntary(self):
        return self.type in [ActionType.Call, ActionType.CallAi, ActionType.Bet, ActionType.BetAi,\
                             ActionType.Raise, ActionType.RaiseAi]

class Player(object): # pylint: disable=too-few-public-methods
    def __init__(self):
        self.name = None
        self.position = None
        self.collected = 0.00

class Hand(object): # pylint: disable=too-many-instance-attributes
    def __init__(self):
        self.id = None
        self.lines = []
        self.players = {}
        self.pot = None
        self.stakes = None
        self.preflop = []
        self.flop = []
        self.turn = []
        self.river = []

    def preflop_actions(self, player_name):
        return [a for a in self.preflop if a.player.name == player_name]

    def flop_actions(self, player_name):
        return [a for a in self.flop if a.player.name == player_name]

    def turn_actions(self, player_name):
        return [a for a in self.turn if a.player.name == player_name]

    def river_actions(self, player_name):
        return [a for a in self.river if a.player.name == player_name]

    def profit_for_player(self, player_name):
        def invested(acc, action):
            if action.is_raise():
                return action.value
            elif action.type == ActionType.Uncalled:
                return acc - action.value
            return acc + action.value

        return self.players[player_name].collected \
                - reduce(invested, self.preflop_actions(player_name), 0) \
                - reduce(invested, self.flop_actions(player_name), 0) \
                - reduce(invested, self.turn_actions(player_name), 0) \
                - reduce(invested, self.river_actions(player_name), 0)

def profit_for_player(hands, player_name):
    return round(reduce(lambda acc, h: acc + h.profit_for_player(player_name), hands, 0), 2)

def is_call_preflop(actions, player_name):
    for action in actions:
        if action.player.name == player_name and action.is_voluntary():
            return action.is_call()
    return False

def is_raise_preflop(actions, player_name):
    for action in actions:
        if action.player.name == player_name and action.is_voluntary():
            return action.is_raise()
    return False

def is_successful_steal_preflop(actions, player_name):
    return not is_3bet_preflop(actions, player_name) and \
           [ActionType.Raise, ActionType.Uncalled] == [a.type for a in actions if a.player.name == player_name]

def is_unsuccessful_steal_preflop(actions, player_name):
    return not is_3bet_preflop(actions, player_name) and \
           [ActionType.Raise, ActionType.Fold] == [a.type for a in actions if a.player.name == player_name]

def is_3bet_preflop(actions, player_name):
    raises = [a for a in actions if a.is_raise()]
    return is_raise_preflop(actions, player_name) and len(raises) >= 2 and \
           raises[0].player.name != player_name and raises[1].player.name == player_name

def is_4bet_preflop(actions, player_name):
    raises = [a for a in actions if a.is_raise()]
    return is_raise_preflop(actions, player_name) and len(raises) >= 3 and \
          raises[2].player.name == player_name



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from codecs import BOM_UTF8
from os import listdir
from os.path import isdir, join
import re
from .entity import Action, ActionType, Hand, Player # pylint: disable=no-name-in-module

class Parser(object): # pylint: disable=too-many-instance-attributes
    def __init__(self):
        self.amt_re = r'\$(\d+(.\d+)?)'
        self.player_re = r'(\S+(\s+\S+)*)'

        # Example:
        # PokerStars Hand #1:  Hold'em No Limit ($0.05/$0.10) - 2017/08/07 23:12:54 CCT [2017/08/07 11:12:54 ET]
        self.game_info_re = re.compile(r'(\w+) Hand #(\d+):\s+(\w.+)\s\(%s/%s\)' \
            % (self.amt_re, self.amt_re))

        # Example:
        # Table 'Aludra' 6-max Seat #1 is the button
        self.table_info_re = re.compile(r'Table \'(.*)\' (\S+)')

        # Example:
        # Seat 1: PLAYER_BTN ($21.05 in chips)
        self.player_info_re = re.compile(r'Seat (\d+): %s \(%s' % (self.player_re, self.amt_re))

        self.table_type_to_player_count = {'6-max': 6, '9-max': 9, '10-max': 10, 'heads-up': 2}
        self.seat_to_position = {'1':'BTN', '2':'SB', '3':'BB', '4':'UTG', '5':'MP', '6':'CO'}

        # Example:
        # Dealt to PLAYER_CO [Ah Ac]
        self.dealt_re = re.compile(r'Dealt to %s \[(.*)\]' % self.player_re)
        self.flop_re = re.compile(r'\*\*\* FLOP \*\*\*\s+\[(.*)\]')
        self.turn_re = re.compile(r'\*\*\* TURN \*\*\*\s+\[.*\]\s+\[(.*)\]')
        self.river_re = re.compile(r'\*\*\* RIVER \*\*\*\s+\[.*\]\s+\[(.*)\]')
        self.pot_re = re.compile(r'Total pot %s \| Rake %s' % (self.amt_re, self.amt_re))

        # Example:
        # PLAYER_SB: posts small blind $0.05
        self.post_re = re.compile(r'%s: posts (small|big) blind %s' % (self.player_re, self.amt_re))
        self.fold_re = re.compile(r'{}: folds'.format(self.player_re))
        self.check_re = re.compile(r'{}: checks'.format(self.player_re))
        self.call_ai_re = re.compile(r'{}: calls \$(\d+(.\d+)?) and is all-in'.format(self.player_re))
        self.call_re = re.compile(r'{}: calls \$(\d+(.\d+)?)'.format(self.player_re))
        self.bet_ai_re = re.compile(r'{}: bets \$(\d+(.\d+)?) and is all-in'.format(self.player_re))
        self.bet_re = re.compile(r'{}: bets \$(\d+(.\d+)?)'.format(self.player_re))
        self.raise_ai_re = re.compile(r'{}: raises \$(\d+(.\d+)?) to \$(\d+(.\d+)?) and is all-in'.format(self.player_re))
        self.raise_re = re.compile(r'{}: raises \$(\d+(.\d+)?) to \$(\d+(.\d+)?)'.format(self.player_re))
        self.uncalled_re = re.compile(r'Uncalled bet \(\$(\d+(.\d+)?)\) returned to {}'.format(self.player_re))
        self.collected_re = re.compile(r'%s collected %s from pot' % (self.player_re, self.amt_re))

    def parse_game_info(self, hand):
        m_res = re.match(self.game_info_re, hand.lines[0].lstrip(BOM_UTF8))
        if m_res != None:
            hand.id = m_res.groups()[1]
            hand.stakes = (float(m_res.groups()[3]), float(m_res.groups()[5]))

    def parse_table_info(self, hand):
        m_res = re.match(self.table_info_re, hand.lines[1])
        if m_res != None:
            # currently not used
            pass

    def parse_player_info(self, hand):
        idx = 2
        for idx in xrange(2, 2 + 6):
            m_res = re.match(self.player_info_re, hand.lines[idx])
            if m_res != None:
                player = Player()
                player.name = m_res.groups()[1].decode('utf-8')
                player.position = self.seat_to_position[m_res.groups()[0]]
                hand.players[player.name] = player
        return idx + 1

    def parse_blind_posts(self, hand, idx):
        m_res = re.match(self.post_re, hand.lines[idx])
        if m_res != None:
            action = Action(ActionType.Post, float(m_res.groups()[3]))
            action.player = hand.players[m_res.groups()[0].decode('utf-8')]
            hand.preflop.append(action)

    def parse_action(self, hand, idx, inserter): # pylint: disable=too-many-locals
        fold_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.Fold))
        check_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.Check))
        call_ai_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.CallAi, float(m[2])))
        call_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.Call, float(m[2])))
        bet_ai_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.BetAi, float(m[2])))
        bet_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.Bet, float(m[2])))
        raise_ai_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.RaiseAi, float(m[4])))
        raise_mod = lambda m: inserter(hand, m[0].decode('utf-8'), Action(ActionType.Raise, float(m[4])))
        uncalled_mod = lambda m: inserter(hand, m[2].decode('utf-8'), Action(ActionType.Uncalled, float(m[0])))
        collected_mod = lambda m: setattr(hand.players[m[0].decode('utf-8')], 'collected', float(m[2]))

        def match_and_return_index(idx, hand, pattern, modifier):
            m_res = re.match(pattern, hand.lines[idx])
            if m_res != None:
                modifier(m_res.groups())
                return idx + 1
            return idx

        def ignore_line(idx, line):
            if 'has timed out' in line or \
                'is disconnected' in line or \
                '*** SHOW DOWN ***' in line or \
                ' shows [' in line or \
                ' mucks hand' in line:
                return idx + 1
            return idx

        while True:
            old_idx = idx
            idx = match_and_return_index(idx, hand, self.fold_re, fold_mod)
            idx = match_and_return_index(idx, hand, self.check_re, check_mod)
            idx = match_and_return_index(idx, hand, self.call_ai_re, call_ai_mod)
            idx = match_and_return_index(idx, hand, self.call_re, call_mod)
            idx = match_and_return_index(idx, hand, self.bet_ai_re, bet_ai_mod)
            idx = match_and_return_index(idx, hand, self.bet_re, bet_mod)
            idx = match_and_return_index(idx, hand, self.raise_ai_re, raise_ai_mod)
            idx = match_and_return_index(idx, hand, self.raise_re, raise_mod)
            idx = match_and_return_index(idx, hand, self.uncalled_re, uncalled_mod)
            idx = match_and_return_index(idx, hand, self.collected_re, collected_mod)
            idx = ignore_line(idx, hand.lines[idx])
            if old_idx == idx:
                break
        return idx

    def parse_preflop_action(self, hand, idx):
        if '*** HOLE CARDS ***' not in hand.lines[idx]:
            return idx
        idx += 1
        m_res = re.match(self.dealt_re, hand.lines[idx])
        while m_res is not None:
            hand.players[m_res.groups()[0].decode('utf-8')].holding = m_res.groups()[2]
            idx += 1
            m_res = re.match(self.dealt_re, hand.lines[idx])

        def insert_action(hand, player, action):
            action.player = hand.players[player]
            hand.preflop.append(action)

        return self.parse_action(hand, idx, insert_action)

    def parse_flop_action(self, hand, idx):
        m_res = re.match(self.flop_re, hand.lines[idx])
        if m_res is None:
            return idx

        def insert_action(hand, player, action):
            action.player = hand.players[player]
            hand.flop.append(action)

        return self.parse_action(hand, idx + 1, insert_action)

    def parse_turn_action(self, hand, idx):
        m_res = re.match(self.turn_re, hand.lines[idx])
        if m_res is None:
            return idx

        def insert_action(hand, player, action):
            action.player = hand.players[player]
            hand.turn.append(action)

        return self.parse_action(hand, idx + 1, insert_action)

    def parse_river_action(self, hand, idx):
        m_res = re.match(self.river_re, hand.lines[idx])
        if m_res is None:
            return idx

        def insert_action(hand, player, action):
            action.player = hand.players[player]
            hand.river.append(action)

        return self.parse_action(hand, idx + 1, insert_action)

    def parse_summary(self, hand, idx):
        for i in xrange(idx, len(hand.lines)):
            if '*** SUMMARY ***' in hand.lines[i]:
                m_res = re.match(self.pot_re, hand.lines[i+1])
                if m_res != None:
                    hand.pot = float(m_res.groups()[0])
                break

    def parse_hand(self, hand):
        self.parse_game_info(hand)
        self.parse_table_info(hand)
        idx = self.parse_player_info(hand)
        self.parse_blind_posts(hand, idx)
        self.parse_blind_posts(hand, idx+1)
        idx = self.parse_preflop_action(hand, idx+2)
        idx = self.parse_flop_action(hand, idx)
        idx = self.parse_turn_action(hand, idx)
        idx = self.parse_river_action(hand, idx)
        self.parse_summary(hand, idx)

    def parse_file_contents(self, lines, store_lines):
        result = []
        hand_in_process = False
        for line in lines:
            if line.strip():
                if not hand_in_process:
                    hand = Hand()
                    hand_in_process = True

                hand.lines.append(line)
            else:
                if hand_in_process:
                    hand.lines[0] = hand.lines[0].replace('PokerStars Zoom Hand', 'PokerStars Hand')
                    hand.lines.append('\r\n')
                    hand.lines.append('\r\n')
                    hand.lines.append('\r\n')
                    self.parse_hand(hand)
                    if hand.pot != None:
                        if not store_lines:
                            hand.lines = None
                        result.append(hand)
                    hand_in_process = False

        return result

def parse_files(file_names, store_lines=False):
    hands = []
    parser = Parser()

    for file_name in file_names:
        if isdir(file_name):
            files_in_dir = [join(file_name, f) for f in listdir(file_name)]
            hands.extend(parse_files(files_in_dir, store_lines))
        else:
            with open(file_name, 'r') as file_desc:
                lines = file_desc.readlines()
            hands.extend(parser.parse_file_contents(lines, store_lines))

    return hands



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from .entity import is_call_preflop, is_raise_preflop, is_3bet_preflop, is_4bet_preflop # pylint: disable=no-name-in-module

def create_player_filter(player_name):
    return lambda h: player_name in h.players.keys()

def create_position_filter(player_name, positions):
    return lambda h: player_name in h.players and h.players[player_name].position in positions

def create_voluntary_filter(player_name, voluntary):
    pred = lambda h: [1 for a in h.preflop + h.flop + h.turn + h.river if a.is_voluntary() and a.player.name == player_name]
    if voluntary == 'only':
        return pred
    return lambda h: not pred(h)

def create_call_pf_filter(player_name):
    return lambda h: is_call_preflop(h.preflop, player_name)

def create_pfr_filter(player_name):
    return lambda h: is_raise_preflop(h.preflop, player_name)

def create_3bet_filter(player_name):
    return lambda h: is_3bet_preflop(h.preflop, player_name)

def create_4bet_filter(player_name):
    return lambda h: is_4bet_preflop(h.preflop, player_name)

def create(hand_filter, player_name):
    result = []

    result.append(create_player_filter(player_name))

    positions = hand_filter.get('positions', None)
    if positions:
        result.append(create_position_filter(player_name, positions))

    voluntary = hand_filter.get('voluntary', None)
    if voluntary:
        result.append(create_voluntary_filter(player_name, voluntary))

    threebet = hand_filter.get('3bet', None)
    if threebet:
        result.append(create_3bet_filter(player_name))

    fourbet = hand_filter.get('4bet', None)
    if fourbet:
        result.append(create_4bet_filter(player_name))

    return result

def apply_filters(hands, filters, sort=False):
    def pred(hand):
        return reduce(lambda acc, flt: acc and flt(hand), filters, True)

    hands = [hand for hand in hands if pred(hand)]
    if sort:
        hands = sorted(hands, key=lambda h: h.pot, reverse=True)
    return hands

def apply_filter(hands, flt):
    return [hand for hand in hands if flt(hand)]



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from flask import Flask, jsonify
from . import hand_parser
from . import report

app = Flask(__name__)
hands = hand_parser.parse_files(['./session'])

@app.route('/api/0.1/position_report/<player_name>', methods=['GET'])
def api_position_report(player_name):
    positions = ['SB', 'BB', 'UTG', 'MP', 'CO', 'BTN']

    result = {}
    for position in positions:
        rep = report.create_position_report(hands, player_name, position)
        result[position] = vars(rep.profit_report)
    result['ALL'] = vars(report.create_profit_report(hands, player_name))

    return jsonify(result)

@app.route('/api/0.1/position_report/<player_name>/<position>', methods=['GET'])
def api_position_detail_report(player_name, position):
    rep = report.create_position_report(hands, player_name, position)
    rep_json = vars(rep)
    rep_json['profit_report'] = vars(rep.profit_report)
    return jsonify(rep_json)

@app.route('/api/0.1/blind_report/<player_name>', methods=['GET'])
def api_blind_report(player_name):
    rep = report.create_blind_report(hands, player_name)
    rep_json = vars(rep)
    rep_json['sb_report'] = vars(rep.sb_report.profit_report)
    rep_json['bb_report'] = vars(rep.bb_report.profit_report)
    return jsonify(rep_json)

@app.route('/api/0.1/preflop_report/<player_name>', methods=['GET'])
def api_preflop_report(player_name):
    rep = report.create_preflop_report(hands, player_name)
    rep_json = vars(rep)
    rep_json['profit_report'] = vars(rep.profit_report)
    return jsonify(rep_json)

if __name__ == '__main__':
    app.run()



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from codecs import BOM_UTF8
from logging import basicConfig, INFO
from sys import stdout

from . import config
from . import hand_filter
from . import hand_parser
from . import report
from . import report_printer

def initialize():
    basicConfig(level=INFO, format='[ %(levelname)s ] %(message)s')
    config.parse_and_validate_args()

def main():
    initialize()

    store_lines = config.action == 'dump_ps'
    hands = hand_parser.parse_files(config.files, store_lines)
    hand_filters = hand_filter.create(config.hand_filter, config.player_name)
    hands = hand_filter.apply_filters(hands, hand_filters, config.sort)

    if config.action == 'dump_ps':
        stdout.write(BOM_UTF8)
        stdout.writelines(reduce(lambda a, h: a + h.lines, hands, []))

    if config.action == 'report':
        report_printer.print_stats(hands, config.player_name)

    if config.action == 'blind_report':
        rep = report.create_blind_report(hands, config.player_name)
        report_printer.print_blind_report(rep)

    if config.action == 'position_report':
        positions = ['SB', 'BB', 'UTG', 'MP', 'CO', 'BTN']
        reports = [report.create_position_report(hands, config.player_name, p) for p in positions]
        for rep in reports:
            report_printer.print_position_report(rep)
        rep = report.create_profit_report(hands, config.player_name)
        report_printer.print_profit_report(rep)

    if config.action == 'preflop_report':
        rep = report.create_preflop_report(hands, config.player_name)
        report_printer.print_preflop_report(rep)

if __name__ == '__main__':
    main()



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

def print_blind_report(report):
    print_position_report(report.sb_report)
    print "Expected profit: {}".format(report.sb_expected_profit)
    print "VPIP profit: {}".format(report.sb_vpip_profit)
    print "Expected VPIP profit: {}".format(report.sb_expected_vpip_profit)
    print "Forced profit: {}".format(report.sb_forced_profit)
    print
    print_position_report(report.bb_report)
    print "Expected profit: {}".format(report.bb_expected_profit)
    print "VPIP profit: {}".format(report.bb_vpip_profit)
    print "Expected VPIP profit: {}".format(report.bb_expected_vpip_profit)
    print "Forced profit: {}".format(report.bb_forced_profit)
    print "Expected forced profit: {}".format(report.bb_expected_forced_profit)
    print

def print_profit_report(report):
    print "Hands played: {}".format(report.hand_count)
    print "VPIP%: {}".format(report.vpip)
    print "PFR%: {}".format(report.pfr)
    print "PFR profit: {}".format(report.pfr_profit)
    print "Flat%: {}".format(report.flat)
    print "Flat profit: {}".format(report.flat_profit)
    print "3bet%: {}".format(report.threebet)
    print "3bet profit: {}".format(report.threebet_profit)
    print "4bet%: {}".format(report.fourbet)
    print "4bet profit: {}".format(report.fourbet_profit)
    print "Total profit: {}".format(report.profit)
    print "Total profit (/100): {}".format(report.profit_per_100)
    print

def print_position_report(report):
    print report.position
    print_profit_report(report.profit_report)

def print_preflop_report(report):
    print_profit_report(report.profit_report)
    print 'Steal success%: {}'.format(report.steal_success)
    print 'Steal fail%: {}'.format(report.steal_fail)
    print 'Steal profit: {}'.format(report.steal_profit)
    print 'Steal profit (/100): {}'.format(report.steal_profit_per_100)
    print

# pylint: skip-file
def print_stats(hands, player):
    for h in []: #hands:
        print(h.lines[0].strip())
        print('PREFLOP')
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.players[player].preflop, "|")))
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.preflop, "|")))

        print('FLOP')
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.players[player].flop, "|")))
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.flop, "|")))

        print('TURN')
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.players[player].turn, "|")))
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.turn, "|")))

        print('RIVER')
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.players[player].river, "|")))
        print('action {}'.format(reduce(lambda acc, a: acc + str(a) + '|', h.river, "|")))

    print('Hand statistics')
    print('Hands: {}'.format(len(hands)))
    positions = ['SB', 'BB', 'UTG', 'MP', 'CO', 'BTN']
    for pos in positions:
        print('{} profit: {}'.format(pos, reduce(lambda acc, h: acc + h.profit_for_player(player), filter(lambda h: h.players[player].position == pos, hands), 0)))
    print('Total profit: {}'.format(reduce(lambda acc, h: acc + h.profit_for_player(player), hands, 0)))
    print('Profit/100: {}'.format(reduce(lambda acc, h: acc + h.profit_for_player(player), hands, 0) * 100 / len(hands)))

    preflop_lines = {}
    flop_lines = {}
    turn_lines = {}
    river_lines = {}
    for h in hands:
        l = reduce(lambda acc, a: acc + a.type.value, h.preflop_actions(player), '')
        preflop_lines[l] = preflop_lines.get(l, 0) + 1
        l = reduce(lambda acc, a: acc + a.type.value, h.flop_actions(player), '')
        flop_lines[l] = flop_lines.get(l, 0) + 1
        l = reduce(lambda acc, a: acc + a.type.value, h.turn_actions(player), '')
        turn_lines[l] = turn_lines.get(l, 0) + 1
        l = reduce(lambda acc, a: acc + a.type.value, h.river_actions(player), '')
        river_lines[l] = river_lines.get(l, 0) + 1

    print('Lines taken (p - post, x - check, c - call, b - bet, r - raise, u - bet uncalled)')
    for (l,c) in preflop_lines.iteritems():
        print('Preflop {}: {}'.format(l, c))
    for (l,c) in flop_lines.iteritems():
        if l != '':
            print('Flop {}: {}'.format(l, c))
    for (l,c) in turn_lines.iteritems():
        if l != '':
            print('Turn {}: {}'.format(l, c))
    for (l,c) in river_lines.iteritems():
        if l != '':
            print('River {}: {}'.format(l, c))



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from .entity import is_successful_steal_preflop, is_unsuccessful_steal_preflop, profit_for_player # pylint: disable=no-name-in-module
from .hand_filter import apply_filter, create_call_pf_filter, create_pfr_filter, create_3bet_filter
from .hand_filter import create_4bet_filter, create_player_filter, create_position_filter, create_voluntary_filter

def div(num1, num2):
    if num2 == 0:
        return 0.00
    return round(float(num1) / num2, 2)

class BlindReport(object): # pylint: disable=too-many-instance-attributes,too-few-public-methods
    def __init__(self):
        self.sb_expected_profit = 0
        self.sb_expected_vpip_profit = 0
        self.sb_vpip_profit = 0
        self.sb_forced_profit = 0
        self.sb_report = None

        self.bb_expected_profit = 0
        self.bb_expected_vpip_profit = 0
        self.bb_vpip_profit = 0
        self.bb_expected_forced_profit = 0
        self.bb_forced_profit = 0
        self.bb_report = None

class ProfitReport(object): # pylint: disable=too-many-instance-attributes,too-few-public-methods
    def __init__(self):
        self.hand_count = 0
        self.profit = 0
        self.profit_per_100 = 0
        self.vpip = 0
        self.pfr = 0
        self.pfr_profit = 0
        self.flat = 0
        self.flat_profit = 0
        self.threebet = 0
        self.threebet_profit = 0
        self.fourbet = 0
        self.fourbet_profit = 0

class PositionReport(object): # pylint: disable=too-many-instance-attributes,too-few-public-methods
    def __init__(self):
        self.position = None
        self.profit_report = 0

class PreflopReport(object): # pylint: disable=too-many-instance-attributes,too-few-public-methods
    def __init__(self):
        self.steal_success = 0
        self.steal_fail = 0
        self.steal_profit = 0
        self.steal_profit_per_100 = 0
        self.profit_report = None

def create_blind_report(hands, player_name):
    report = BlindReport()

    sb_hands = apply_filter(hands, create_position_filter(player_name, ['SB']))
    sb_voluntary_hands = apply_filter(sb_hands, create_voluntary_filter(player_name, 'only'))
    sb_forced_hands = apply_filter(sb_hands, create_voluntary_filter(player_name, 'forced'))

    report.sb_report = create_position_report(hands, player_name, 'SB')
    report.sb_expected_profit = round(-sum([h.stakes[0] for h in sb_hands]), 2)
    report.sb_expected_vpip_profit = round(-sum([h.stakes[0] for h in sb_voluntary_hands]), 2)
    report.sb_vpip_profit = profit_for_player(sb_voluntary_hands, player_name)
    report.sb_forced_profit = profit_for_player(sb_forced_hands, player_name)

    bb_hands = apply_filter(hands, create_position_filter(player_name, ['BB']))
    bb_voluntary_hands = apply_filter(bb_hands, create_voluntary_filter(player_name, 'only'))
    bb_forced_hands = apply_filter(bb_hands, create_voluntary_filter(player_name, 'forced'))

    report.bb_report = create_position_report(hands, player_name, 'BB')
    report.bb_expected_profit = round(-sum([h.stakes[1] for h in bb_hands]), 2)
    report.bb_expected_vpip_profit = round(-sum([h.stakes[1] for h in bb_voluntary_hands]), 2)
    report.bb_vpip_profit = profit_for_player(bb_voluntary_hands, player_name)
    report.bb_expected_forced_profit = round(-sum([h.stakes[1] for h in bb_forced_hands]), 2)
    report.bb_forced_profit = profit_for_player(bb_forced_hands, player_name)

    return report

def create_profit_report(hands, player_name):
    report = ProfitReport()

    hands = apply_filter(hands, create_player_filter(player_name))
    hands_len = len(hands)
    voluntary_hands = apply_filter(hands, create_voluntary_filter(player_name, 'only'))
    pfr_hands = apply_filter(voluntary_hands, create_pfr_filter(player_name))
    flat_hands = apply_filter(voluntary_hands, create_call_pf_filter(player_name))
    threebet_hands = apply_filter(voluntary_hands, create_3bet_filter(player_name))
    fourbet_hands = apply_filter(voluntary_hands, create_4bet_filter(player_name))

    report.hand_count = hands_len
    if report.hand_count == 0:
        return report

    report.profit = profit_for_player(hands, player_name)
    report.profit_per_100 = div(report.profit * 100, hands_len)
    report.vpip = div(len(voluntary_hands) * 100, hands_len)
    report.pfr = div(len(pfr_hands) * 100, hands_len)
    report.pfr_profit = profit_for_player(pfr_hands, player_name)
    report.flat = div(len(flat_hands) * 100, hands_len)
    report.flat_profit = profit_for_player(flat_hands, player_name)
    report.threebet = div(len(threebet_hands) * 100, hands_len)
    report.threebet_profit = profit_for_player(threebet_hands, player_name)
    report.fourbet = div(len(fourbet_hands) * 100, hands_len)
    report.fourbet_profit = profit_for_player(fourbet_hands, player_name)

    return report

def create_position_report(hands, player_name, position):
    report = PositionReport()

    hands = apply_filter(hands, create_position_filter(player_name, [position]))
    report.position = position
    report.profit_report = create_profit_report(hands, player_name)

    return report

def create_preflop_report(hands, player_name):
    report = PreflopReport()

    pfr_hands = apply_filter(hands, create_pfr_filter(player_name))
    pfr_hands_len = len(pfr_hands)
    stolen_pot_hands = apply_filter(pfr_hands, lambda h: is_successful_steal_preflop(h.preflop, player_name))
    failed_steal_hands = apply_filter(pfr_hands, lambda h: is_unsuccessful_steal_preflop(h.preflop, player_name))

    report.steal_success = div(len(stolen_pot_hands) * 100, pfr_hands_len)
    report.steal_fail = div(len(failed_steal_hands) * 100, pfr_hands_len)
    report.steal_profit = profit_for_player(stolen_pot_hands + failed_steal_hands, player_name)
    report.steal_profit_per_100 = div(report.steal_profit * 100, pfr_hands_len)
    report.profit_report = create_profit_report(hands, player_name)

    return report



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

setup(name='poker_stats',
      version='0.1.0.dev3',
      author='Polish Poker Community in Manila',
      author_email='dev@null.com',
      url='https://github.com/yarpenzigrin/pokerstats',
      description='Statistics generator for poker hands played online',
      long_description='Statistics generator for poker hands played online\nUnder construction',
      license='MIT',
      classifiers=[
          'Development Status :: 2 - Pre-Alpha',
          'Intended Audience :: End Users/Desktop',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2.7'],
      packages=find_packages(exclude=['test', 'test.ut']),
      entry_points={'console_scripts':['poker_stats = poker_stats:main']},
      python_requires='>=2.7, <3',
      install_requires=['enum34==1.1.6', 'pyparsing==2.1.10', 'Flask==0.12.2', 'gunicorn==19.7.1']
)



#!/usr/bin/env python2.7
# -*- coding: utf-8 -*-

from argparse import ArgumentParser
from pyparsing import Literal, Word, StringEnd, Suppress, ZeroOrMore

action = None
files = []
hand_filter = {}
player_name = None
sort = False

def parse_filter(line):
    single_position = Literal('SB') ^ Literal('BB') ^ Literal('UTG') ^ Literal('MP') ^ Literal('CO') ^ Literal('BTN')
    position_list = single_position + ZeroOrMore(Suppress(Word(',')) + single_position)
    position = (Literal('pos') ^ Literal('position')) + Suppress('=') + position_list('positions')
    voluntary = Literal('voluntary') + Suppress('=') + (Literal('only') ^ Literal('forced'))('voluntary')
    threebet = Literal('3bet')('3bet')
    fourbet = Literal('4bet')('4bet')
    anyf = (position ^ voluntary ^ threebet ^ fourbet)
    grammar = anyf + ZeroOrMore(Suppress(Word(';')) + anyf) + StringEnd()

    return grammar.parseString(line).asDict()

def parse_args():
    parser = ArgumentParser()
    parser.add_argument('-f', '--filter', help='Hand filter e. g. --filter "position=BTN,CO;voluntary=forced"')
    action_parser = parser.add_subparsers(help='Available actions', dest='action')

    dump_parser = action_parser.add_parser('dump_ps', help='Dump hands in PS format')
    dump_parser.add_argument('-s', '--sort', action='store_true', help='Sort the dump by the size of the pot')
    dump_parser.add_argument('player_name', help='Player nickname')
    dump_parser.add_argument('files', help='File list', nargs='+')

    report_parser = action_parser.add_parser('report', help='Print hand report for a player')
    report_parser.add_argument('player_name', help='Player nickname')
    report_parser.add_argument('files', help='File list', nargs='+')

    blind_report_parser = action_parser.add_parser('blind_report', help='Print report for a player about play on the blinds')
    blind_report_parser.add_argument('player_name', help='Player nickname')
    blind_report_parser.add_argument('files', help='File list', nargs='+')

    position_report_parser = action_parser.add_parser('position_report', help='Print positional report for a player')
    position_report_parser.add_argument('player_name', help='Player nickname')
    position_report_parser.add_argument('files', help='File list', nargs='+')

    preflop_report_parser = action_parser.add_parser('preflop_report', help='Print report for a player about preflop play')
    preflop_report_parser.add_argument('player_name', help='Player nickname')
    preflop_report_parser.add_argument('files', help='File list', nargs='+')

    return parser.parse_args()

def parse_and_validate_args():
    global action, files, player_name, sort
    args = parse_args()
    action = args.action
    files = args.files
    if args.__contains__('sort') and args.sort:
        sort = args.sort
    if args.__contains__('filter') and args.filter:
        hand_filter.update(parse_filter(args.filter))
    if args.__contains__('player_name') and args.player_name:
        player_name = args.player_name
